#manager-row {
    margin-left: 2px;
    margin-right: 2px;
    margin-bottom: 4px;
}

#local-manager,
#remote-manager {
    padding-left: 0px;
    padding-right: 0px;
}

.header-fixed {
    table-layout: fixed;
    margin-bottom: 0px;
    width: 100%;
}

tr {
    width: 100%;
    display: inline-table;
}

td {
    padding-top: 4px;
    padding-bottom: 4px;
}

div.outer {
    position: relative;
    height: 22px;
}

div.inner {
    position: absolute;
    width: 100%;
    white-space: nowrap;
    overflow: hidden;
    overflow: ellipsis;
}

.header-fixed > thead {
    display: block;
    /*margin-right: 15px;*/
}

.header-fixed > tbody {
    display: block;
    overflow-y: auto;
    width: 100%;
}

th {
    cursor: pointer;
}

th#num,
td#num {
    min-width: 50px;
}

th#ctime,
td#ctime {
    min-width: 143px;
}

th#mtime,
td#mtime {
    min-width: 143px;
}

th#name,
td#name {
    min-width: 80px;
}

th#size,
td#size {
    min-width: 80px;
}

th#type,
td#type {
    min-width: 50px;
}

button.btn-operation {
    padding-top: 0px;
    padding-bottom: 0px;
    margin-left: 5px;
}

.span-pre {
    display: block;
    unicode-bidi: embed;
    font-family: monospace;
    white-space: pre;
}

#log-row {
    margin-left: 2px;
    margin-right: 2px;
}

input.dir-path:focus {
    outline: none !important;
    border-color: grey;
    box-shadow: none;
}

#log-console-container {
    width: 100%;
    margin-left: 2px;
    margin-right: 2px;
}

#log-console {
    overflow-y: scroll;
    width: 100%;
    resize: none;
    font-family: monospace;
    white-space: pre;
}

#log-console:focus {
    outline: none !important;
    border-color: grey;
    box-shadow: none;
}
